from __future__ import unicode_literals, absolute_import

ENCODING = 'cp1252'
MSG_PREFIX = b'SAMP'

# Opcode definitions
OPCODE_INFO = b'i'
OPCODE_RULES = b'r'
OPCODE_CLIENTS = b'c'
OPCODE_CLIENTS_DETAILED = b'd'
OPCODE_RCON = b'x'
OPCODE_PSEUDORANDOM = b'p'

RCON_CMDLIST = b'cmdlist'
RCON_VARLIST = b'varlist'
RCON_EXIT = b'exit'
RCON_ECHO = b'echo'
RCON_HOSTNAME = b'hostname'
RCON_GAMEMODETEXT = b'gamemodetext'
RCON_MAPNAME = b'mapname'
RCON_EXEC = b'exec'
RCON_KICK = b'kick'
RCON_BAN = b'ban'
RCON_CHANGEMODE = b'changemode'
RCON_GMX = b'gmx'
RCON_RELOADBANS = b'reloadbans'
RCON_RELOADLOG = b'reloadlog'
RCON_SAY = b'say'
RCON_PLAYERS = b'players'
RCON_BANIP = b'banip'
RCON_UNBANIP = b'unbanip'
RCON_GRAVITY = b'gravity'
RCON_WEATHER = b'weather'
RCON_LOADFS = b'loadfs'
RCON_WEBURL = b'weburl'
RCON_UNLOADFS = b'unloadfs'
RCON_RELOADFS = b'reloadfs'
RCON_RCON_PASSWORD = b'rcon_password'
RCON_PASSWORD = b'password'
RCON_MESSAGESLIMIT = b'messageslimit'
RCON_ACKSLIMIT = b'ackslimit'
RCON_MESSAGEHOLELIMIT = b'messageholelimit'
RCON_PLAYERTIMEOUT = b'playertimeout'
RCON_LANGUAGE = b'language'
